# Copyright (C) 2000-2018 by the Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

import sys
import time
import locale
import gettext
from types import StringType, UnicodeType

from Mailman import mm_cfg
from Mailman.Utils import GetCharSet, IsLanguage
from Mailman.SafeDict import SafeDict

_translation = None
_ctranslation = None


if gettext.NullTranslations.__dict__.has_key('output_charset'): 
    _NullTranslations = gettext.NullTranslations
    _gettranslation = gettext.translation
else:
    # wrapping old gettext module classes and translation()
    class _NullTranslations(gettext.NullTranslations):
        def __init__(self, fp=None):
            gettext.NullTranslations.__init__(self, fp)
            self._ocharset = None

        def output_charset(self):
            return self._ocharset

        def set_output_charset(self, charset):
            self._ocharset = charset

    class _GNUTranslations(gettext.GNUTranslations):
        def __init__(self, fp=None):
            gettext.GNUTranslations.__init__(self, fp)
            self._ocharset = None

        def output_charset(self):
            return self._ocharset

        def set_output_charset(self, charset):
            self._ocharset = charset

        def gettext(self, message):
            s = gettext.GNUTranslations.gettext(self, message)
            if (self._ocharset and self._charset 
                and self._ocharset != self._charset):
                s = s.decode(self._charset).encode(self._ocharset)
            return s

    def _gettranslation(domain, localedir=None, languages=None,
                    class_=None, fallback=False, codeset=None):
        if class_ is None:
            cls = _GNUTranslations
        else:
            cls = class_
        trs = gettext.translation(domain, localedir, languages,
                                   cls, fallback)
        if trs.__class__ == gettext.NullTranslations:
            return _NullTranslations() 
        if codeset is not None:
            trs.set_output_charset(codeset)
        return trs


def _get_ctype_charset():
    try:
        old = locale.setlocale(locale.LC_CTYPE, '')
        charset = locale.nl_langinfo(locale.CODESET)
        locale.setlocale(locale.LC_CTYPE, old)
    except locale.Error:
        # should fall back to 'C' locale...
        charset = None
    return charset


def set_language(language=None):
    global _translation
    cset = None
    if language is not None:
        if IsLanguage(language):
            cset = GetCharSet(language)
        language = language.split('.')
        if len(language) == 2:
            if cset is None:
                cset = language[1]
            language = language[:1] 
    try:
        if cset is None:
            _translation = _gettranslation('mailman', mm_cfg.MESSAGES_DIR,
                                           language, fallback = True)
        else:
            _translation = _gettranslation('mailman', mm_cfg.MESSAGES_DIR,
                                           language, fallback = True,
                                           codeset=cset)
    except IOError:
        # The selected language was not installed in messages, so fall back to
        # untranslated English.
        _translation = _NullTranslations()

def get_translation():
    return _translation

def set_translation(translation):
    global _translation
    _translation = translation


# Set up the global translation based on environment variables.  Mostly used
# for command line scripts if DISABLE_COMMAND_LOCALE_CSET is yes
if _translation is None:
    set_language()


# Set up the translation for command line scripts based on environment
# variables. 
if not mm_cfg.DISABLE_COMMAND_LOCALE_CSET:
    _ctype_charset = _get_ctype_charset()
    if _ctranslation is None:
        if _ctype_charset is not None:
            _ctranslation = _gettranslation('mailman', mm_cfg.MESSAGES_DIR,
                                            fallback = True, 
                                            codeset=_ctype_charset)
        else:
            _ctranslation = _NullTranslations()
else:
    _ctype_charset = None


def _(s, frame=1, translation=None):
    if s == '':
        return s
    assert s
    # Do translation of the given string into the current language, and do
    # Ping-string interpolation into the resulting string.
    #
    # This lets you write something like:
    #
    #     now = time.ctime(time.time())
    #     print _('The current time is: %(now)s')
    #
    # and have it Just Work.  Note that the lookup order for keys in the
    # original string is 1) locals dictionary, 2) globals dictionary.
    #
    # First, get the frame of the caller
    frame = sys._getframe(frame)
    # A `safe' dictionary is used so we won't get an exception if there's a
    # missing key in the dictionary.
    dict = SafeDict(frame.f_globals.copy())
    dict.update(frame.f_locals)
    # Translating the string returns an encoded 8-bit string.  Rather than
    # turn that into a Unicode, we turn any Unicodes in the dictionary values
    # into encoded 8-bit strings.  BAW: Returning a Unicode here broke too
    # much other stuff and _() has many tentacles.  Eventually I think we want
    # to use Unicode everywhere.
    if not translation: 
        translation = _translation
    tns = translation.gettext(s)
    charset = (translation.output_charset() or translation.charset()
               or 'us-ascii')
    for k, v in dict.items():
        if isinstance(v, UnicodeType):
            dict[k] = v.encode(charset, 'replace')
    try:
        return tns % dict
    except (ValueError, TypeError):
        # Bad interpolation format. Punt.
        return tns



def tolocale(s):
    global _ctype_charset
    if isinstance(s, UnicodeType) or _ctype_charset in (None, 'US-ASCII'):
        return s
    source = _translation.output_charset() or _translation.charset()
    if source in (None, 'US-ASCII') or source == _ctype_charset:
        return s
    return unicode(s, source, 'replace').encode(_ctype_charset, 'replace')

if mm_cfg.DISABLE_COMMAND_LOCALE_CSET:
    C_ = _
else:
    def C_(s):
        global _ctranslation
        try:
            rs = _(s, 2, _ctranslation)
        except UnicodeEncodeError:
            rs = _(s, 2)
        return rs



def ctime(date):
    # Don't make these module globals since we have to do runtime translation
    # of the strings anyway.
    daysofweek = [
        _('Mon'), _('Tue'), _('Wed'), _('Thu'),
        _('Fri'), _('Sat'), _('Sun')
        ]
    months = [
        '',
        _('Jan'), _('Feb'), _('Mar'), _('Apr'), _('May'), _('Jun'),
        _('Jul'), _('Aug'), _('Sep'), _('Oct'), _('Nov'), _('Dec')
        ]

    tzname = _('Server Local Time')
    if isinstance(date, StringType):
        try:
            year, mon, day, hh, mm, ss, wday, ydat, dst = time.strptime(date)
            if dst in (0,1):
                tzname = time.tzname[dst]
            else:
                # MAS: No exception but dst = -1 so try
                return ctime(time.mktime((year, mon, day, hh, mm, ss, wday,
                                          ydat, dst)))
        except (ValueError, AttributeError):
            try:
                wday, mon, day, hms, year = date.split()
                hh, mm, ss = hms.split(':')
                year = int(year)
                day = int(day)
                hh = int(hh)
                mm = int(mm)
                ss = int(ss)
            except ValueError:
                return date
            else:
                for i in range(0, 7):
                    wconst = (1999, 1, 1, 0, 0, 0, i, 1, 0)
                    if wday.lower() == time.strftime('%a', wconst).lower():
                        wday = i
                        break
                for i in range(1, 13):
                    mconst = (1999, i, 1, 0, 0, 0, 0, 1, 0)
                    if mon.lower() == time.strftime('%b', mconst).lower():
                        mon = i
                        break
    else:
        year, mon, day, hh, mm, ss, wday, yday, dst = time.localtime(date)
        if dst in (0,1):
            tzname = time.tzname[dst]

    wday = daysofweek[wday]
    mon = months[mon]
    return _('%(wday)s %(mon)s %(day)2i %(hh)02i:%(mm)02i:%(ss)02i '
             '%(tzname)s %(year)04i')
